@@ -14,10 +14,11 @@
 // limitations under the License.
 //
 
-#include <grpc/support/port_platform.h>
-
 #include "src/core/ext/filters/rbac/rbac_service_config_parser.h"
 
+#include <grpc/grpc_audit_logging.h>
+#include <grpc/support/port_platform.h>
+
 #include <cstdint>
 #include <map>
 #include <memory>
@@ -27,14 +28,11 @@
 #include "absl/status/statusor.h"
 #include "absl/strings/str_cat.h"
 #include "absl/types/optional.h"
-
-#include <grpc/grpc_audit_logging.h>
-
 #include "src/core/lib/channel/channel_args.h"
-#include "src/core/lib/matchers/matchers.h"
 #include "src/core/lib/security/authorization/audit_logging.h"
 #include "src/core/util/json/json_args.h"
 #include "src/core/util/json/json_object_loader.h"
+#include "src/core/util/matchers.h"
 
 namespace grpc_core {
 
@@ -355,6 +353,13 @@ void RbacConfig::RbacPolicy::Rules::Policy::HeaderMatch::JsonPostLoad(
                                              range_match->end, invert_match));
     return;
   }
+  auto string_match = LoadJsonObjectField<StringMatch>(
+      json.object(), args, "stringMatch", errors, /*required=*/false);
+  if (string_match.has_value()) {
+    matcher = HeaderMatcher::CreateFromStringMatcher(
+        name, std::move(string_match->matcher), invert_match);
+    return;
+  }
   if (errors->size() == original_error_size) {
     errors->AddError("no valid matcher found");
   }
@@ -392,7 +397,8 @@ void RbacConfig::RbacPolicy::Rules::Policy::StringMatch::JsonPostLoad(
                                                   field_name, errors,
                                                   /*required=*/false);
     if (match.has_value()) {
-      set_string_matcher(StringMatcher::Create(type, *match, ignore_case));
+      set_string_matcher(
+          StringMatcher::Create(type, *match, /*case_sensitive=*/!ignore_case));
       return true;
     }
     return false;
@@ -408,7 +414,7 @@ void RbacConfig::RbacPolicy::Rules::Policy::StringMatch::JsonPostLoad(
                                                          /*required=*/false);
   if (regex_match.has_value()) {
     set_string_matcher(StringMatcher::Create(StringMatcher::Type::kSafeRegex,
-                                             regex_match->regex, ignore_case));
+                                             regex_match->regex));
     return;
   }
   if (errors->size() == original_error_size) {
