@@ -15,25 +15,19 @@
 #ifndef GRPC_SRC_CORE_EXT_TRANSPORT_CHAOTIC_GOOD_CLIENT_CHAOTIC_GOOD_CONNECTOR_H
 #define GRPC_SRC_CORE_EXT_TRANSPORT_CHAOTIC_GOOD_CLIENT_CHAOTIC_GOOD_CONNECTOR_H
 
+#include <grpc/event_engine/event_engine.h>
+#include <grpc/support/port_platform.h>
+
 #include <cstddef>
 #include <cstdint>
 #include <memory>
 
 #include "absl/random/random.h"
 #include "absl/status/statusor.h"
-
-#include <grpc/event_engine/event_engine.h>
-#include <grpc/support/port_platform.h>
-
 #include "src/core/client_channel/connector.h"
-#include "src/core/ext/transport/chttp2/transport/hpack_encoder.h"
-#include "src/core/ext/transport/chttp2/transport/hpack_parser.h"
 #include "src/core/handshaker/handshaker.h"
 #include "src/core/lib/channel/channel_args.h"
 #include "src/core/lib/event_engine/channel_args_endpoint_config.h"
-#include "src/core/lib/gprpp/notification.h"
-#include "src/core/lib/gprpp/ref_counted_ptr.h"
-#include "src/core/lib/gprpp/sync.h"
 #include "src/core/lib/iomgr/closure.h"
 #include "src/core/lib/iomgr/endpoint.h"
 #include "src/core/lib/iomgr/error.h"
@@ -45,6 +39,9 @@
 #include "src/core/lib/resource_quota/memory_quota.h"
 #include "src/core/lib/resource_quota/resource_quota.h"
 #include "src/core/lib/transport/promise_endpoint.h"
+#include "src/core/util/notification.h"
+#include "src/core/util/ref_counted_ptr.h"
+#include "src/core/util/sync.h"
 
 namespace grpc_core {
 namespace chaotic_good {
@@ -68,15 +65,15 @@ class ChaoticGoodConnector : public SubchannelConnector {
 
  private:
   static auto DataEndpointReadSettingsFrame(
-      RefCountedPtr<ChaoticGoodConnector> self);
+      RefCountedPtr<ChaoticGoodConnector> self, uint32_t data_connection_index);
   static auto DataEndpointWriteSettingsFrame(
-      RefCountedPtr<ChaoticGoodConnector> self);
+      RefCountedPtr<ChaoticGoodConnector> self, uint32_t data_connection_index);
   static auto ControlEndpointReadSettingsFrame(
       RefCountedPtr<ChaoticGoodConnector> self);
   static auto ControlEndpointWriteSettingsFrame(
       RefCountedPtr<ChaoticGoodConnector> self);
-  static auto WaitForDataEndpointSetup(
-      RefCountedPtr<ChaoticGoodConnector> self);
+  static auto WaitForDataEndpointSetup(RefCountedPtr<ChaoticGoodConnector> self,
+                                       uint32_t data_connection_index);
   void OnHandshakeDone(absl::StatusOr<HandshakerArgs*> result);
 
   RefCountedPtr<Arena> arena_ = SimpleArenaAllocator()->MakeArena();
@@ -89,16 +86,13 @@ class ChaoticGoodConnector : public SubchannelConnector {
       resolved_addr_;
 
   PromiseEndpoint control_endpoint_;
-  PromiseEndpoint data_endpoint_;
+  std::vector<PromiseEndpoint> data_endpoints_;
+  std::vector<std::string> connection_ids_;
   ActivityPtr connect_activity_ ABSL_GUARDED_BY(mu_);
   const std::shared_ptr<grpc_event_engine::experimental::EventEngine>
       event_engine_;
   RefCountedPtr<HandshakeManager> handshake_mgr_;
-  HPackCompressor hpack_compressor_;
-  HPackParser hpack_parser_;
-  absl::BitGen bitgen_;
-  InterActivityLatch<void> data_endpoint_ready_;
-  std::string connection_id_;
+  std::vector<std::unique_ptr<InterActivityLatch<void>>> data_endpoint_ready_;
 };
 }  // namespace chaotic_good
 }  // namespace grpc_core
