@@ -18,9 +18,8 @@
 #include <utility>
 
 #include "absl/strings/str_cat.h"
-
-#include "src/core/lib/gprpp/crash.h"  // IWYU pragma: keep
 #include "src/core/lib/iomgr/port.h"
+#include "src/core/util/crash.h"  // IWYU pragma: keep
 
 #ifdef GRPC_POSIX_WAKEUP_FD
 #include <errno.h>
@@ -31,7 +30,7 @@
 #endif
 
 #include "src/core/lib/event_engine/posix_engine/wakeup_fd_pipe.h"
-#include "src/core/lib/gprpp/strerror.h"
+#include "src/core/util/strerror.h"
 
 namespace grpc_event_engine {
 namespace experimental {
