@@ -11,18 +11,17 @@
 // WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 // See the License for the specific language governing permissions and
 // limitations under the License.
-#include <grpc/support/port_platform.h>
-
 #include "src/core/lib/event_engine/time_util.h"
 
-#include <chrono>
-
 #include <grpc/event_engine/event_engine.h>
+#include <grpc/support/port_platform.h>
+
+#include <chrono>
 
 namespace grpc_event_engine {
 namespace experimental {
 
-int64_t Milliseconds(EventEngine::Duration d) {
+size_t Milliseconds(EventEngine::Duration d) {
   return std::chrono::duration_cast<std::chrono::milliseconds>(d).count();
 }
 
