@@ -15,20 +15,21 @@
 
 #ifdef GPR_WINDOWS
 
-#include "absl/cleanup/cleanup.h"
-#include "absl/functional/any_invocable.h"
-#include "absl/status/status.h"
-#include "absl/strings/str_format.h"
-
 #include <grpc/event_engine/memory_allocator.h>
 #include <grpc/support/log_windows.h>
 
+#include "absl/cleanup/cleanup.h"
+#include "absl/functional/any_invocable.h"
+#include "absl/log/check.h"
+#include "absl/log/log.h"
+#include "absl/status/status.h"
+#include "absl/strings/str_format.h"
 #include "src/core/lib/event_engine/tcp_socket_utils.h"
-#include "src/core/lib/event_engine/trace.h"
+#include "src/core/lib/event_engine/thread_pool/thread_pool.h"
 #include "src/core/lib/event_engine/windows/windows_endpoint.h"
-#include "src/core/lib/gprpp/debug_location.h"
-#include "src/core/lib/gprpp/status_helper.h"
 #include "src/core/lib/iomgr/error.h"
+#include "src/core/util/debug_location.h"
+#include "src/core/util/status_helper.h"
 
 namespace grpc_event_engine {
 namespace experimental {
@@ -40,8 +41,9 @@ constexpr int kMaxWSABUFCount = 16;
 void DumpSliceBuffer(SliceBuffer* buffer, absl::string_view context_string) {
   for (size_t i = 0; i < buffer->Count(); i++) {
     auto str = buffer->MutableSliceAt(i).as_string_view();
-    gpr_log(GPR_INFO, "%s: %.*s", context_string.data(), str.length(),
-            str.data());
+    GRPC_TRACE_LOG(event_engine_endpoint, INFO)
+        << context_string << " [" << i + 1 << "/" << buffer->Count()
+        << "]: " << str;
   }
 }
 
@@ -50,13 +52,12 @@ void DumpSliceBuffer(SliceBuffer* buffer, absl::string_view context_string) {
 WindowsEndpoint::WindowsEndpoint(
     const EventEngine::ResolvedAddress& peer_address,
     std::unique_ptr<WinSocket> socket, MemoryAllocator&& allocator,
-    const EndpointConfig& /* config */, Executor* executor,
+    const EndpointConfig& /* config */, ThreadPool* thread_pool,
     std::shared_ptr<EventEngine> engine)
     : peer_address_(peer_address),
       allocator_(std::move(allocator)),
-      executor_(executor),
-      io_state_(std::make_shared<AsyncIOState>(this, std::move(socket),
-                                               std::move(engine))) {
+      io_state_(std::make_shared<AsyncIOState>(
+          this, std::move(socket), std::move(engine), thread_pool)) {
   char addr[EventEngine::ResolvedAddress::MAX_SIZE_BYTES];
   int addr_len = sizeof(addr);
   if (getsockname(io_state_->socket->raw_socket(),
@@ -73,16 +74,20 @@ WindowsEndpoint::WindowsEndpoint(
 
 WindowsEndpoint::~WindowsEndpoint() {
   io_state_->socket->Shutdown(DEBUG_LOCATION, "~WindowsEndpoint");
-  GRPC_EVENT_ENGINE_ENDPOINT_TRACE("~WindowsEndpoint::%p", this);
+  GRPC_TRACE_LOG(event_engine_endpoint, INFO) << "~WindowsEndpoint::" << this;
 }
 
-absl::Status WindowsEndpoint::DoTcpRead(SliceBuffer* buffer) {
-  GRPC_EVENT_ENGINE_ENDPOINT_TRACE("WindowsEndpoint::%p reading", this);
-  if (io_state_->socket->IsShutdown()) {
-    return absl::UnavailableError("Socket is shutting down.");
+void WindowsEndpoint::AsyncIOState::DoTcpRead(SliceBuffer* buffer) {
+  GRPC_TRACE_LOG(event_engine_endpoint, INFO)
+      << "WindowsEndpoint::" << endpoint << " attempting a read";
+  if (socket->IsShutdown()) {
+    socket->read_info()->SetErrorStatus(
+        absl::InternalError("Socket is shutting down."));
+    thread_pool->Run(&handle_read_event);
+    return;
   }
   // Prepare the WSABUF struct
-  GPR_ASSERT(buffer->Count() <= kMaxWSABUFCount);
+  CHECK(buffer->Count() <= kMaxWSABUFCount);
   WSABUF wsa_buffers[kMaxWSABUFCount];
   for (size_t i = 0; i < buffer->Count(); i++) {
     auto& slice = buffer->MutableSliceAt(i);
@@ -93,41 +98,39 @@ absl::Status WindowsEndpoint::DoTcpRead(SliceBuffer* buffer) {
   DWORD flags = 0;
   // First try a synchronous, non-blocking read.
   int status =
-      WSARecv(io_state_->socket->raw_socket(), wsa_buffers,
-              (DWORD)buffer->Count(), &bytes_read, &flags, nullptr, nullptr);
+      WSARecv(socket->raw_socket(), wsa_buffers, (DWORD)buffer->Count(),
+              &bytes_read, &flags, nullptr, nullptr);
   int wsa_error = status == 0 ? 0 : WSAGetLastError();
   if (wsa_error != WSAEWOULDBLOCK) {
     // Data or some error was returned immediately.
-    io_state_->socket->read_info()->SetResult(
-        {/*wsa_error=*/wsa_error, /*bytes_read=*/bytes_read});
-    executor_->Run(&io_state_->handle_read_event);
-    return absl::OkStatus();
+    socket->read_info()->SetResult(wsa_error, bytes_read, "WSARecv");
+    thread_pool->Run(&handle_read_event);
+    return;
   }
   // If the endpoint has already received some data, and the next call would
   // block, return the data in case that is all the data the reader expects.
-  if (io_state_->handle_read_event.MaybeFinishIfDataHasAlreadyBeenRead()) {
-    return absl::OkStatus();
+  if (handle_read_event.MaybeFinishIfDataHasAlreadyBeenRead()) {
+    return;
   }
   // Otherwise, let's retry, by queuing a read.
-  status = WSARecv(io_state_->socket->raw_socket(), wsa_buffers,
-                   (DWORD)buffer->Count(), &bytes_read, &flags,
-                   io_state_->socket->read_info()->overlapped(), nullptr);
+  socket->NotifyOnRead(&handle_read_event);
+  status = WSARecv(socket->raw_socket(), wsa_buffers, (DWORD)buffer->Count(),
+                   nullptr, &flags, socket->read_info()->overlapped(), nullptr);
   wsa_error = status == 0 ? 0 : WSAGetLastError();
   if (wsa_error != 0 && wsa_error != WSA_IO_PENDING) {
-    // Async read returned immediately with an error
-    return GRPC_WSA_ERROR(
-        wsa_error,
-        absl::StrFormat("WindowsEndpont::%p Read failed", this).c_str());
+    // The async read attempt returned an error immediately.
+    socket->UnregisterReadCallback();
+    socket->read_info()->SetResult(
+        wsa_error, 0, absl::StrFormat("WindowsEndpoint::%p Read failed", this));
+    thread_pool->Run(&handle_read_event);
   }
-  io_state_->socket->NotifyOnRead(&io_state_->handle_read_event);
-  return absl::OkStatus();
 }
 
 bool WindowsEndpoint::Read(absl::AnyInvocable<void(absl::Status)> on_read,
                            SliceBuffer* buffer, const ReadArgs* /* args */) {
   if (io_state_->socket->IsShutdown()) {
-    executor_->Run([on_read = std::move(on_read)]() mutable {
-      on_read(absl::UnavailableError("Socket is shutting down."));
+    io_state_->thread_pool->Run([on_read = std::move(on_read)]() mutable {
+      on_read(absl::InternalError("Socket is shutting down."));
     });
     return false;
   }
@@ -140,37 +143,34 @@ bool WindowsEndpoint::Read(absl::AnyInvocable<void(absl::Status)> on_read,
     buffer->AppendIndexed(Slice(allocator_.MakeSlice(min_read_size)));
   }
   io_state_->handle_read_event.Prime(io_state_, buffer, std::move(on_read));
-  auto status = DoTcpRead(buffer);
-  if (!status.ok()) {
-    // The read could not be completed.
-    io_state_->endpoint->executor_->Run([this, status]() {
-      io_state_->handle_read_event.ExecuteCallbackAndReset(status);
-    });
-  }
+  io_state_->DoTcpRead(buffer);
   return false;
 }
 
 bool WindowsEndpoint::Write(absl::AnyInvocable<void(absl::Status)> on_writable,
                             SliceBuffer* data, const WriteArgs* /* args */) {
-  GRPC_EVENT_ENGINE_ENDPOINT_TRACE("WindowsEndpoint::%p writing", this);
+  GRPC_TRACE_LOG(event_engine_endpoint, INFO)
+      << "WindowsEndpoint::" << this << " writing";
   if (io_state_->socket->IsShutdown()) {
-    executor_->Run([on_writable = std::move(on_writable)]() mutable {
-      on_writable(absl::UnavailableError("Socket is shutting down."));
-    });
+    io_state_->thread_pool->Run(
+        [on_writable = std::move(on_writable)]() mutable {
+          on_writable(absl::InternalError("Socket is shutting down."));
+        });
     return false;
   }
-  if (grpc_event_engine_endpoint_data_trace.enabled()) {
+  if (GRPC_TRACE_FLAG_ENABLED(event_engine_endpoint_data)) {
     for (size_t i = 0; i < data->Count(); i++) {
       auto str = data->RefSlice(i).as_string_view();
-      gpr_log(GPR_INFO, "WindowsEndpoint::%p WRITE (peer=%s): %.*s", this,
-              peer_address_string_.c_str(), str.length(), str.data());
+      GRPC_TRACE_LOG(event_engine_endpoint, INFO)
+          << "WindowsEndpoint::" << this
+          << " WRITE (peer=" << peer_address_string_ << "): " << str;
     }
   }
-  GPR_ASSERT(data->Count() <= UINT_MAX);
+  CHECK(data->Count() <= UINT_MAX);
   absl::InlinedVector<WSABUF, kMaxWSABUFCount> buffers(data->Count());
   for (size_t i = 0; i < data->Count(); i++) {
     auto& slice = data->MutableSliceAt(i);
-    GPR_ASSERT(slice.size() <= ULONG_MAX);
+    CHECK(slice.size() <= ULONG_MAX);
     buffers[i].len = slice.size();
     buffers[i].buf = (char*)slice.begin();
   }
@@ -182,7 +182,7 @@ bool WindowsEndpoint::Write(absl::AnyInvocable<void(absl::Status)> on_writable,
   if (status == 0) {
     if (bytes_sent == data->Length()) {
       // Write completed, exiting early
-      executor_->Run(
+      io_state_->thread_pool->Run(
           [cb = std::move(on_writable)]() mutable { cb(absl::OkStatus()); });
       return false;
     }
@@ -203,13 +203,16 @@ bool WindowsEndpoint::Write(absl::AnyInvocable<void(absl::Status)> on_writable,
     // then we can avoid doing an async write operation at all.
     int wsa_error = WSAGetLastError();
     if (wsa_error != WSAEWOULDBLOCK) {
-      executor_->Run([cb = std::move(on_writable), wsa_error]() mutable {
-        cb(GRPC_WSA_ERROR(wsa_error, "WSASend"));
-      });
+      io_state_->thread_pool->Run(
+          [cb = std::move(on_writable), wsa_error]() mutable {
+            cb(GRPC_WSA_ERROR(wsa_error, "WSASend"));
+          });
       return false;
     }
   }
   auto write_info = io_state_->socket->write_info();
+  io_state_->handle_write_event.Prime(io_state_, data, std::move(on_writable));
+  io_state_->socket->NotifyOnWrite(&io_state_->handle_write_event);
   status =
       WSASend(io_state_->socket->raw_socket(), &buffers[async_buffers_offset],
               (DWORD)(data->Count() - async_buffers_offset), nullptr, 0,
@@ -217,16 +220,11 @@ bool WindowsEndpoint::Write(absl::AnyInvocable<void(absl::Status)> on_writable,
   if (status != 0) {
     int wsa_error = WSAGetLastError();
     if (wsa_error != WSA_IO_PENDING) {
-      executor_->Run([cb = std::move(on_writable), wsa_error]() mutable {
-        cb(GRPC_WSA_ERROR(wsa_error, "WSASend"));
-      });
-      return false;
+      io_state_->socket->UnregisterWriteCallback();
+      io_state_->socket->write_info()->SetResult(wsa_error, 0, "WSASend");
+      io_state_->thread_pool->Run(&io_state_->handle_write_event);
     }
   }
-  // As all is now setup, we can now ask for the IOCP notification. It may
-  // trigger the callback immediately however, but no matter.
-  io_state_->handle_write_event.Prime(io_state_, data, std::move(on_writable));
-  io_state_->socket->NotifyOnWrite(&io_state_->handle_write_event);
   return false;
 }
 const EventEngine::ResolvedAddress& WindowsEndpoint::GetPeerAddress() const {
@@ -245,16 +243,22 @@ void AbortOnEvent(absl::Status) {
 }
 }  // namespace
 
-void WindowsEndpoint::HandleReadClosure::Reset() {
-  io_state_.reset();
+absl::AnyInvocable<void(absl::Status)>
+WindowsEndpoint::HandleReadClosure::ResetAndReturnCallback() {
+  auto cb = std::move(cb_);
   cb_ = &AbortOnEvent;
   buffer_ = nullptr;
+  io_state_.reset();
+  return cb;
 }
 
-void WindowsEndpoint::HandleWriteClosure::Reset() {
-  io_state_.reset();
+absl::AnyInvocable<void(absl::Status)>
+WindowsEndpoint::HandleWriteClosure::ResetAndReturnCallback() {
+  auto cb = std::move(cb_);
   cb_ = &AbortOnEvent;
   buffer_ = nullptr;
+  io_state_.reset();
+  return cb;
 }
 
 void WindowsEndpoint::HandleReadClosure::Prime(
@@ -275,98 +279,108 @@ void WindowsEndpoint::HandleWriteClosure::Prime(
 
 void WindowsEndpoint::HandleReadClosure::Run() {
   // Deletes the shared_ptr when this closure returns
+  // Note that the endpoint may have already been destroyed.
   auto io_state = std::move(io_state_);
-  GRPC_EVENT_ENGINE_ENDPOINT_TRACE("WindowsEndpoint::%p Handling Read Event",
-                                   io_state->endpoint);
-  absl::Status status;
+  GRPC_TRACE_LOG(event_engine_endpoint, INFO)
+      << "WindowsEndpoint::" << io_state->endpoint << " Handling Read Event";
   const auto result = io_state->socket->read_info()->result();
+  if (!result.error_status.ok()) {
+    buffer_->Clear();
+    return ResetAndReturnCallback()(result.error_status);
+  }
+  absl::Status status;
   if (result.wsa_error != 0) {
     status = GRPC_WSA_ERROR(result.wsa_error, "Async Read Error");
     buffer_->Clear();
-    return ExecuteCallbackAndReset(status);
+    return ResetAndReturnCallback()(status);
   }
   if (result.bytes_transferred == 0) {
+    DCHECK_GT(io_state.use_count(), 0);
     // Either the endpoint is shut down or we've seen the end of the stream
-    if (grpc_event_engine_endpoint_data_trace.enabled()) {
+    if (GRPC_TRACE_FLAG_ENABLED(event_engine_endpoint_data)) {
+      LOG(INFO) << "WindowsEndpoint::" << this << " read 0 bytes.";
       DumpSliceBuffer(
-          buffer_, absl::StrFormat("WindowsEndpoint::%p READ (peer=%s)",
-                                   io_state->endpoint,
-                                   io_state->endpoint->peer_address_string_));
+          &last_read_buffer_,
+          absl::StrFormat("WindowsEndpoint::%p READ last_read_buffer_: ",
+                          io_state->endpoint));
     }
-    status = absl::UnavailableError("End of TCP stream");
-    grpc_core::StatusSetInt(&status, grpc_core::StatusIntProperty::kRpcStatus,
-                            GRPC_STATUS_UNAVAILABLE);
     buffer_->Swap(last_read_buffer_);
-    return ExecuteCallbackAndReset(status);
+    if (buffer_->Length() == 0) {
+      // Only send an error if there is no more data to consume. If the endpoint
+      // or socket is shut down, the next read will discover that.
+      status = absl::InternalError("End of TCP stream");
+      grpc_core::StatusSetInt(&status, grpc_core::StatusIntProperty::kRpcStatus,
+                              GRPC_STATUS_UNAVAILABLE);
+    }
+    return ResetAndReturnCallback()(status);
   }
-  GPR_DEBUG_ASSERT(result.bytes_transferred > 0);
-  GPR_DEBUG_ASSERT(result.bytes_transferred <= buffer_->Length());
+  DCHECK_GT(result.bytes_transferred, 0);
+  DCHECK(result.bytes_transferred <= buffer_->Length());
   buffer_->MoveFirstNBytesIntoSliceBuffer(result.bytes_transferred,
                                           last_read_buffer_);
   if (buffer_->Length() == 0) {
     buffer_->Swap(last_read_buffer_);
-    return ExecuteCallbackAndReset(status);
+    return ResetAndReturnCallback()(status);
   }
   // Doing another read. Let's keep the AsyncIOState alive a bit longer.
   io_state_ = std::move(io_state);
-  status = io_state_->endpoint->DoTcpRead(buffer_);
-  if (!status.ok()) {
-    io_state_.reset();
-    ExecuteCallbackAndReset(status);
-  }
+  io_state_->DoTcpRead(buffer_);
 }
 
 bool WindowsEndpoint::HandleReadClosure::MaybeFinishIfDataHasAlreadyBeenRead() {
   if (last_read_buffer_.Length() > 0) {
+    GRPC_TRACE_LOG(event_engine_endpoint, INFO)
+        << "WindowsEndpoint::" << io_state_->endpoint
+        << " finishing a synchronous read";
     buffer_->Swap(last_read_buffer_);
-    io_state_->endpoint->executor_->Run(
-        [this]() { ExecuteCallbackAndReset(absl::OkStatus()); });
+    if (GRPC_TRACE_FLAG_ENABLED(event_engine_endpoint_data)) {
+      DumpSliceBuffer(buffer_, "finishing synchronous read");
+    }
+    io_state_->thread_pool->Run(
+        [cb = ResetAndReturnCallback()]() mutable { cb(absl::OkStatus()); });
     return true;
   }
   return false;
 }
 
-void WindowsEndpoint::HandleReadClosure::ExecuteCallbackAndReset(
-    absl::Status status) {
-  auto cb = std::move(cb_);
-  Reset();
-  cb(status);
-}
-
 void WindowsEndpoint::HandleReadClosure::DonateSpareSlices(
     SliceBuffer* buffer) {
   // Donee buffer must be empty.
-  GPR_ASSERT(buffer->Length() == 0);
+  CHECK_EQ(buffer->Length(), 0);
   // HandleReadClosure must be in the reset state.
-  GPR_ASSERT(buffer_ == nullptr);
+  CHECK_EQ(buffer_, nullptr);
   buffer->Swap(last_read_buffer_);
 }
 
 void WindowsEndpoint::HandleWriteClosure::Run() {
   // Deletes the shared_ptr when this closure returns
   auto io_state = std::move(io_state_);
-  GRPC_EVENT_ENGINE_ENDPOINT_TRACE("WindowsEndpoint::%p Handling Write Event",
-                                   io_state->endpoint);
-  auto cb = std::move(cb_);
+  GRPC_TRACE_LOG(event_engine_endpoint, INFO)
+      << "WindowsEndpoint::" << io_state->endpoint << " Handling Write Event";
   const auto result = io_state->socket->write_info()->result();
+  if (!result.error_status.ok()) {
+    buffer_->Clear();
+    return ResetAndReturnCallback()(result.error_status);
+  }
   absl::Status status;
   if (result.wsa_error != 0) {
     status = GRPC_WSA_ERROR(result.wsa_error, "WSASend");
   } else {
-    GPR_ASSERT(result.bytes_transferred == buffer_->Length());
+    CHECK(result.bytes_transferred == buffer_->Length());
   }
-  Reset();
-  cb(status);
+  return ResetAndReturnCallback()(status);
 }
 
 // ---- AsyncIOState ----
 
 WindowsEndpoint::AsyncIOState::AsyncIOState(WindowsEndpoint* endpoint,
                                             std::unique_ptr<WinSocket> socket,
-                                            std::shared_ptr<EventEngine> engine)
+                                            std::shared_ptr<EventEngine> engine,
+                                            ThreadPool* thread_pool)
     : endpoint(endpoint),
       socket(std::move(socket)),
-      engine(std::move(engine)) {}
+      engine(std::move(engine)),
+      thread_pool(thread_pool) {}
 
 WindowsEndpoint::AsyncIOState::~AsyncIOState() {
   socket->Shutdown(DEBUG_LOCATION, "~AsyncIOState");
