@@ -16,7 +16,6 @@
 #define GRPC_SRC_CORE_LIB_PROMISE_INTER_ACTIVITY_PIPE_H
 
 #include <grpc/support/port_platform.h>
-
 #include <stdint.h>
 
 #include <array>
@@ -24,12 +23,11 @@
 
 #include "absl/base/thread_annotations.h"
 #include "absl/types/optional.h"
-
-#include "src/core/lib/gprpp/ref_counted.h"
-#include "src/core/lib/gprpp/ref_counted_ptr.h"
-#include "src/core/lib/gprpp/sync.h"
 #include "src/core/lib/promise/activity.h"
 #include "src/core/lib/promise/poll.h"
+#include "src/core/util/ref_counted.h"
+#include "src/core/util/ref_counted_ptr.h"
+#include "src/core/util/sync.h"
 
 namespace grpc_core {
 
