@@ -12,22 +12,23 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include <grpc/support/port_platform.h>
-
 #include "src/core/lib/promise/sleep.h"
 
+#include <grpc/event_engine/event_engine.h>
+#include <grpc/support/port_platform.h>
+
 #include <utility>
 
-#include <grpc/event_engine/event_engine.h>
-
-#include "src/core/lib/event_engine/default_event_engine.h"
-#include "src/core/lib/gprpp/time.h"
+#include "src/core/lib/event_engine/event_engine_context.h"  // IWYU pragma: keep
 #include "src/core/lib/iomgr/exec_ctx.h"
 #include "src/core/lib/promise/activity.h"
+#include "src/core/lib/promise/context.h"
+#include "src/core/lib/promise/poll.h"
+#include "src/core/util/time.h"
 
 namespace grpc_core {
 
-using ::grpc_event_engine::experimental::GetDefaultEventEngine;
+using ::grpc_event_engine::experimental::EventEngine;
 
 Sleep::Sleep(Timestamp deadline) : deadline_(deadline) {}
 
@@ -39,26 +40,28 @@ Poll<absl::Status> Sleep::operator()() {
   // Invalidate now so that we see a fresh version of the time.
   // TODO(ctiller): the following can be safely removed when we remove ExecCtx.
   ExecCtx::Get()->InvalidateNow();
+  const auto now = Timestamp::Now();
   // If the deadline is earlier than now we can just return.
-  if (deadline_ <= ExecCtx::Get()->Now()) return absl::OkStatus();
+  if (deadline_ <= now) return absl::OkStatus();
   if (closure_ == nullptr) {
     // TODO(ctiller): it's likely we'll want a pool of closures - probably per
     // cpu? - to avoid allocating/deallocating on fast paths.
     closure_ = new ActiveClosure(deadline_);
   }
+  if (closure_->HasRun()) return absl::OkStatus();
   return Pending{};
 }
 
 Sleep::ActiveClosure::ActiveClosure(Timestamp deadline)
-    : waker_(Activity::current()->MakeOwningWaker()),
-      timer_handle_(GetDefaultEventEngine()->RunAfter(
-          deadline - ExecCtx::Get()->Now(), this)) {}
+    : waker_(GetContext<Activity>()->MakeOwningWaker()),
+      timer_handle_(GetContext<EventEngine>()->RunAfter(
+          deadline - Timestamp::Now(), this)) {}
 
 void Sleep::ActiveClosure::Run() {
   ApplicationCallbackExecCtx callback_exec_ctx;
   ExecCtx exec_ctx;
   auto waker = std::move(waker_);
-  if (refs_.Unref()) {
+  if (Unref()) {
     delete this;
   } else {
     waker.Wakeup();
@@ -69,9 +72,19 @@ void Sleep::ActiveClosure::Cancel() {
   // If we cancel correctly then we must own both refs still and can simply
   // delete without unreffing twice, otherwise try unreffing since this may be
   // the last owned ref.
-  if (GetDefaultEventEngine()->Cancel(timer_handle_) || refs_.Unref()) {
+  if (HasRun() || GetContext<EventEngine>()->Cancel(timer_handle_) || Unref()) {
     delete this;
   }
 }
 
+bool Sleep::ActiveClosure::Unref() {
+  return (refs_.fetch_sub(1, std::memory_order_acq_rel) == 1);
+}
+
+bool Sleep::ActiveClosure::HasRun() const {
+  // If the closure has run (ie woken up the activity) then it will have
+  // decremented this ref count once.
+  return refs_.load(std::memory_order_acquire) == 1;
+}
+
 }  // namespace grpc_core
