@@ -16,16 +16,15 @@
 #define GRPC_SRC_CORE_LIB_PROMISE_TRY_SEQ_H
 
 #include <grpc/support/port_platform.h>
-
 #include <stdlib.h>
 
 #include <type_traits>
 #include <utility>
 
+#include "absl/log/check.h"
 #include "absl/meta/type_traits.h"
 #include "absl/status/status.h"
 #include "absl/status/statusor.h"
-
 #include "src/core/lib/promise/detail/basic_seq.h"
 #include "src/core/lib/promise/detail/promise_like.h"
 #include "src/core/lib/promise/detail/seq_state.h"
@@ -42,22 +41,21 @@ struct TrySeqTraitsWithSfinae {
   using UnwrappedType = T;
   using WrappedType = absl::StatusOr<T>;
   template <typename Next>
-  static auto CallFactory(Next* next, T&& value) {
+  GPR_ATTRIBUTE_ALWAYS_INLINE_FUNCTION static auto CallFactory(Next* next,
+                                                               T&& value) {
     return next->Make(std::forward<T>(value));
   }
-  static bool IsOk(const T&) { return true; }
+  GPR_ATTRIBUTE_ALWAYS_INLINE_FUNCTION static bool IsOk(const T&) {
+    return true;
+  }
   static const char* ErrorString(const T&) { abort(); }
   template <typename R>
   static R ReturnValue(T&&) {
     abort();
   }
-  template <typename F, typename Elem>
-  static auto CallSeqFactory(F& f, Elem&& elem, T&& value)
-      -> decltype(f(std::forward<Elem>(elem), std::forward<T>(value))) {
-    return f(std::forward<Elem>(elem), std::forward<T>(value));
-  }
   template <typename Result, typename RunNext>
-  static Poll<Result> CheckResultAndRunNext(T prior, RunNext run_next) {
+  GPR_ATTRIBUTE_ALWAYS_INLINE_FUNCTION static Poll<Result>
+  CheckResultAndRunNext(T prior, RunNext run_next) {
     return run_next(std::move(prior));
   }
 };
@@ -67,25 +65,25 @@ struct TrySeqTraitsWithSfinae<absl::StatusOr<T>> {
   using UnwrappedType = T;
   using WrappedType = absl::StatusOr<T>;
   template <typename Next>
-  static auto CallFactory(Next* next, absl::StatusOr<T>&& status) {
+  GPR_ATTRIBUTE_ALWAYS_INLINE_FUNCTION static auto CallFactory(
+      Next* next, absl::StatusOr<T>&& status) {
     return next->Make(std::move(*status));
   }
-  static bool IsOk(const absl::StatusOr<T>& status) { return status.ok(); }
+  GPR_ATTRIBUTE_ALWAYS_INLINE_FUNCTION static bool IsOk(
+      const absl::StatusOr<T>& status) {
+    return status.ok();
+  }
   static std::string ErrorString(const absl::StatusOr<T>& status) {
     return status.status().ToString();
   }
   template <typename R>
-  static R ReturnValue(absl::StatusOr<T>&& status) {
+  GPR_ATTRIBUTE_ALWAYS_INLINE_FUNCTION static R ReturnValue(
+      absl::StatusOr<T>&& status) {
     return FailureStatusCast<R>(status.status());
   }
-  template <typename F, typename Elem>
-  static auto CallSeqFactory(F& f, Elem&& elem, absl::StatusOr<T> value)
-      -> decltype(f(std::forward<Elem>(elem), std::move(*value))) {
-    return f(std::forward<Elem>(elem), std::move(*value));
-  }
   template <typename Result, typename RunNext>
-  static Poll<Result> CheckResultAndRunNext(absl::StatusOr<T> prior,
-                                            RunNext run_next) {
+  GPR_ATTRIBUTE_ALWAYS_INLINE_FUNCTION static Poll<Result>
+  CheckResultAndRunNext(absl::StatusOr<T> prior, RunNext run_next) {
     if (!prior.ok()) return FailureStatusCast<Result>(prior.status());
     return run_next(std::move(prior));
   }
@@ -127,19 +125,24 @@ struct TrySeqTraitsWithSfinae<
   using UnwrappedType = void;
   using WrappedType = T;
   template <typename Next>
-  static auto CallFactory(Next* next, T&&) {
+  GPR_ATTRIBUTE_ALWAYS_INLINE_FUNCTION static auto CallFactory(Next* next,
+                                                               T&&) {
     return next->Make();
   }
-  static bool IsOk(const T& status) { return IsStatusOk(status); }
-  static std::string ErrorString(const T& status) {
+  GPR_ATTRIBUTE_ALWAYS_INLINE_FUNCTION static bool IsOk(const T& status) {
+    return IsStatusOk(status);
+  }
+  GPR_ATTRIBUTE_ALWAYS_INLINE_FUNCTION static std::string ErrorString(
+      const T& status) {
     return IsStatusOk(status) ? "OK" : "FAILED";
   }
   template <typename R>
-  static R ReturnValue(T&& status) {
+  GPR_ATTRIBUTE_ALWAYS_INLINE_FUNCTION static R ReturnValue(T&& status) {
     return FailureStatusCast<R>(std::move(status));
   }
   template <typename Result, typename RunNext>
-  static Poll<Result> CheckResultAndRunNext(T prior, RunNext run_next) {
+  GPR_ATTRIBUTE_ALWAYS_INLINE_FUNCTION static Poll<Result>
+  CheckResultAndRunNext(T prior, RunNext run_next) {
     if (!IsStatusOk(prior)) return Result(std::move(prior));
     return run_next(std::move(prior));
   }
@@ -153,20 +156,25 @@ struct TrySeqTraitsWithSfinae<
   using UnwrappedType = decltype(TakeValue(std::declval<T>()));
   using WrappedType = T;
   template <typename Next>
-  static auto CallFactory(Next* next, T&& status) {
+  GPR_ATTRIBUTE_ALWAYS_INLINE_FUNCTION static auto CallFactory(Next* next,
+                                                               T&& status) {
     return next->Make(TakeValue(std::forward<T>(status)));
   }
-  static bool IsOk(const T& status) { return IsStatusOk(status); }
-  static std::string ErrorString(const T& status) {
+  GPR_ATTRIBUTE_ALWAYS_INLINE_FUNCTION static bool IsOk(const T& status) {
+    return IsStatusOk(status);
+  }
+  GPR_ATTRIBUTE_ALWAYS_INLINE_FUNCTION static std::string ErrorString(
+      const T& status) {
     return IsStatusOk(status) ? "OK" : "FAILED";
   }
   template <typename R>
-  static R ReturnValue(T&& status) {
-    GPR_DEBUG_ASSERT(!IsStatusOk(status));
+  GPR_ATTRIBUTE_ALWAYS_INLINE_FUNCTION static R ReturnValue(T&& status) {
+    DCHECK(!IsStatusOk(status));
     return FailureStatusCast<R>(status.status());
   }
   template <typename Result, typename RunNext>
-  static Poll<Result> CheckResultAndRunNext(T prior, RunNext run_next) {
+  GPR_ATTRIBUTE_ALWAYS_INLINE_FUNCTION static Poll<Result>
+  CheckResultAndRunNext(T prior, RunNext run_next) {
     if (!IsStatusOk(prior)) return Result(std::move(prior));
     return run_next(std::move(prior));
   }
@@ -176,20 +184,25 @@ struct TrySeqTraitsWithSfinae<absl::Status> {
   using UnwrappedType = void;
   using WrappedType = absl::Status;
   template <typename Next>
-  static auto CallFactory(Next* next, absl::Status&&) {
+  GPR_ATTRIBUTE_ALWAYS_INLINE_FUNCTION static auto CallFactory(Next* next,
+                                                               absl::Status&&) {
     return next->Make();
   }
-  static bool IsOk(const absl::Status& status) { return status.ok(); }
+  GPR_ATTRIBUTE_ALWAYS_INLINE_FUNCTION static bool IsOk(
+      const absl::Status& status) {
+    return status.ok();
+  }
   static std::string ErrorString(const absl::Status& status) {
     return status.ToString();
   }
   template <typename R>
-  static R ReturnValue(absl::Status&& status) {
+  GPR_ATTRIBUTE_ALWAYS_INLINE_FUNCTION static R ReturnValue(
+      absl::Status&& status) {
     return FailureStatusCast<R>(std::move(status));
   }
   template <typename Result, typename RunNext>
-  static Poll<Result> CheckResultAndRunNext(absl::Status prior,
-                                            RunNext run_next) {
+  GPR_ATTRIBUTE_ALWAYS_INLINE_FUNCTION static Poll<Result>
+  CheckResultAndRunNext(absl::Status prior, RunNext run_next) {
     if (!prior.ok()) return StatusCast<Result>(std::move(prior));
     return run_next(std::move(prior));
   }
@@ -201,34 +214,44 @@ using TrySeqTraits = TrySeqTraitsWithSfinae<T>;
 template <typename P, typename... Fs>
 class TrySeq {
  public:
-  explicit TrySeq(P&& promise, Fs&&... factories, DebugLocation whence)
+  GPR_ATTRIBUTE_ALWAYS_INLINE_FUNCTION explicit TrySeq(P&& promise,
+                                                       Fs&&... factories,
+                                                       DebugLocation whence)
       : state_(std::forward<P>(promise), std::forward<Fs>(factories)...,
                whence) {}
 
-  auto operator()() { return state_.PollOnce(); }
+  GPR_ATTRIBUTE_ALWAYS_INLINE_FUNCTION auto operator()() {
+    return state_.PollOnce();
+  }
 
  private:
   SeqState<TrySeqTraits, P, Fs...> state_;
 };
 
-template <typename I, typename F, typename Arg>
-struct TrySeqIterTraits {
-  using Iter = I;
-  using Factory = F;
-  using Argument = Arg;
-  using IterValue = decltype(*std::declval<Iter>());
-  using StateCreated = decltype(std::declval<F>()(std::declval<IterValue>(),
-                                                  std::declval<Arg>()));
-  using State = PromiseLike<StateCreated>;
-  using Wrapped = typename State::Result;
-
-  using Traits = TrySeqTraits<Wrapped>;
-};
-
 template <typename Iter, typename Factory, typename Argument>
-struct TrySeqIterResultTraits {
-  using IterTraits = TrySeqIterTraits<Iter, Factory, Argument>;
-  using Result = BasicSeqIter<IterTraits>;
+using TrySeqIter = BasicSeqIter<TrySeqTraits, Iter, Factory, Argument>;
+
+template <typename Container, typename Factory, typename Argument>
+struct TrySeqContainerResultTraits {
+  using BaseResult =
+      TrySeqIter<typename Container::iterator, Factory, Argument>;
+  class Result {
+   public:
+    Result(Container container, Factory factory, Argument argument)
+        : container_(std::move(container)),
+          base_result_(container_.begin(), container_.end(), std::move(factory),
+                       std::move(argument)) {}
+    Result(const Result&) = delete;
+    Result& operator=(const Result&) = delete;
+    Result(Result&&) = default;
+    Result& operator=(Result&&) = default;
+
+    auto operator()() { return base_result_(); }
+
+   private:
+    Container container_;
+    BaseResult base_result_;
+  };
 };
 
 }  // namespace promise_detail
@@ -246,33 +269,35 @@ struct TrySeqIterResultTraits {
 // Status to indicate only success/failure. In the case of returning Status,
 // the construction functors take no arguments.
 template <typename F>
-F TrySeq(F functor) {
+GPR_ATTRIBUTE_ALWAYS_INLINE_FUNCTION inline F TrySeq(F functor) {
   return functor;
 }
 
 template <typename F0, typename F1>
-promise_detail::TrySeq<F0, F1> TrySeq(F0 f0, F1 f1, DebugLocation whence = {}) {
+GPR_ATTRIBUTE_ALWAYS_INLINE_FUNCTION inline promise_detail::TrySeq<F0, F1>
+TrySeq(F0 f0, F1 f1, DebugLocation whence = {}) {
   return promise_detail::TrySeq<F0, F1>(std::move(f0), std::move(f1), whence);
 }
 
 template <typename F0, typename F1, typename F2>
-promise_detail::TrySeq<F0, F1, F2> TrySeq(F0 f0, F1 f1, F2 f2,
-                                          DebugLocation whence = {}) {
+GPR_ATTRIBUTE_ALWAYS_INLINE_FUNCTION inline promise_detail::TrySeq<F0, F1, F2>
+TrySeq(F0 f0, F1 f1, F2 f2, DebugLocation whence = {}) {
   return promise_detail::TrySeq<F0, F1, F2>(std::move(f0), std::move(f1),
                                             std::move(f2), whence);
 }
 
 template <typename F0, typename F1, typename F2, typename F3>
-promise_detail::TrySeq<F0, F1, F2, F3> TrySeq(F0 f0, F1 f1, F2 f2, F3 f3,
-                                              DebugLocation whence = {}) {
+GPR_ATTRIBUTE_ALWAYS_INLINE_FUNCTION inline promise_detail::TrySeq<F0, F1, F2,
+                                                                   F3>
+TrySeq(F0 f0, F1 f1, F2 f2, F3 f3, DebugLocation whence = {}) {
   return promise_detail::TrySeq<F0, F1, F2, F3>(
       std::move(f0), std::move(f1), std::move(f2), std::move(f3), whence);
 }
 
 template <typename F0, typename F1, typename F2, typename F3, typename F4>
-promise_detail::TrySeq<F0, F1, F2, F3, F4> TrySeq(F0 f0, F1 f1, F2 f2, F3 f3,
-                                                  F4 f4,
-                                                  DebugLocation whence = {}) {
+GPR_ATTRIBUTE_ALWAYS_INLINE_FUNCTION inline promise_detail::TrySeq<F0, F1, F2,
+                                                                   F3, F4>
+TrySeq(F0 f0, F1 f1, F2 f2, F3 f3, F4 f4, DebugLocation whence = {}) {
   return promise_detail::TrySeq<F0, F1, F2, F3, F4>(
       std::move(f0), std::move(f1), std::move(f2), std::move(f3), std::move(f4),
       whence);
@@ -280,8 +305,10 @@ promise_detail::TrySeq<F0, F1, F2, F3, F4> TrySeq(F0 f0, F1 f1, F2 f2, F3 f3,
 
 template <typename F0, typename F1, typename F2, typename F3, typename F4,
           typename F5>
-promise_detail::TrySeq<F0, F1, F2, F3, F4, F5> TrySeq(
-    F0 f0, F1 f1, F2 f2, F3 f3, F4 f4, F5 f5, DebugLocation whence = {}) {
+GPR_ATTRIBUTE_ALWAYS_INLINE_FUNCTION
+    promise_detail::TrySeq<F0, F1, F2, F3, F4, F5>
+    TrySeq(F0 f0, F1 f1, F2 f2, F3 f3, F4 f4, F5 f5,
+           DebugLocation whence = {}) {
   return promise_detail::TrySeq<F0, F1, F2, F3, F4, F5>(
       std::move(f0), std::move(f1), std::move(f2), std::move(f3), std::move(f4),
       std::move(f5), whence);
@@ -289,9 +316,10 @@ promise_detail::TrySeq<F0, F1, F2, F3, F4, F5> TrySeq(
 
 template <typename F0, typename F1, typename F2, typename F3, typename F4,
           typename F5, typename F6>
-promise_detail::TrySeq<F0, F1, F2, F3, F4, F5, F6> TrySeq(
-    F0 f0, F1 f1, F2 f2, F3 f3, F4 f4, F5 f5, F6 f6,
-    DebugLocation whence = {}) {
+GPR_ATTRIBUTE_ALWAYS_INLINE_FUNCTION
+    promise_detail::TrySeq<F0, F1, F2, F3, F4, F5, F6>
+    TrySeq(F0 f0, F1 f1, F2 f2, F3 f3, F4 f4, F5 f5, F6 f6,
+           DebugLocation whence = {}) {
   return promise_detail::TrySeq<F0, F1, F2, F3, F4, F5, F6>(
       std::move(f0), std::move(f1), std::move(f2), std::move(f3), std::move(f4),
       std::move(f5), std::move(f6), whence);
@@ -299,9 +327,10 @@ promise_detail::TrySeq<F0, F1, F2, F3, F4, F5, F6> TrySeq(
 
 template <typename F0, typename F1, typename F2, typename F3, typename F4,
           typename F5, typename F6, typename F7>
-promise_detail::TrySeq<F0, F1, F2, F3, F4, F5, F6, F7> TrySeq(
-    F0 f0, F1 f1, F2 f2, F3 f3, F4 f4, F5 f5, F6 f6, F7 f7,
-    DebugLocation whence = {}) {
+GPR_ATTRIBUTE_ALWAYS_INLINE_FUNCTION
+    promise_detail::TrySeq<F0, F1, F2, F3, F4, F5, F6, F7>
+    TrySeq(F0 f0, F1 f1, F2 f2, F3 f3, F4 f4, F5 f5, F6 f6, F7 f7,
+           DebugLocation whence = {}) {
   return promise_detail::TrySeq<F0, F1, F2, F3, F4, F5, F6, F7>(
       std::move(f0), std::move(f1), std::move(f2), std::move(f3), std::move(f4),
       std::move(f5), std::move(f6), std::move(f7), whence);
@@ -316,12 +345,19 @@ promise_detail::TrySeq<F0, F1, F2, F3, F4, F5, F6, F7> TrySeq(
 //   }
 //   return argument;
 template <typename Iter, typename Factory, typename Argument>
-typename promise_detail::TrySeqIterResultTraits<Iter, Factory, Argument>::Result
-TrySeqIter(Iter begin, Iter end, Argument argument, Factory factory) {
+GPR_ATTRIBUTE_ALWAYS_INLINE_FUNCTION auto TrySeqIter(Iter begin, Iter end,
+                                                     Argument argument,
+                                                     Factory factory) {
+  return promise_detail::TrySeqIter<Iter, Factory, Argument>(
+      begin, end, std::move(factory), std::move(argument));
+}
+
+template <typename Container, typename Factory, typename Argument>
+auto TrySeqContainer(Container container, Argument argument, Factory factory) {
   using Result =
-      typename promise_detail::TrySeqIterResultTraits<Iter, Factory,
-                                                      Argument>::Result;
-  return Result(begin, end, std::move(factory), std::move(argument));
+      typename promise_detail::TrySeqContainerResultTraits<Container, Factory,
+                                                           Argument>::Result;
+  return Result(std::move(container), std::move(factory), std::move(argument));
 }
 
 }  // namespace grpc_core
