@@ -14,30 +14,28 @@
 
 #include "src/core/lib/security/authorization/grpc_authorization_policy_provider.h"
 
+#include <grpc/grpc_security.h>
+#include <grpc/slice.h>
+#include <grpc/status.h>
+#include <grpc/support/port_platform.h>
+#include <grpc/support/string_util.h>
+#include <grpc/support/time.h>
 #include <stdint.h>
 
 #include <utility>
 
 #include "absl/log/check.h"
+#include "absl/log/log.h"
 #include "absl/types/optional.h"
-
-#include <grpc/grpc_security.h>
-#include <grpc/slice.h>
-#include <grpc/status.h>
-#include <grpc/support/log.h>
-#include <grpc/support/port_platform.h>
-#include <grpc/support/string_util.h>
-#include <grpc/support/time.h>
-
 #include "src/core/lib/debug/trace.h"
-#include "src/core/lib/gprpp/load_file.h"
-#include "src/core/lib/gprpp/status_helper.h"
 #include "src/core/lib/iomgr/error.h"
 #include "src/core/lib/security/authorization/grpc_authorization_engine.h"
 #include "src/core/lib/security/authorization/rbac_policy.h"
 #include "src/core/lib/security/authorization/rbac_translator.h"
 #include "src/core/lib/slice/slice.h"
 #include "src/core/lib/slice/slice_internal.h"
+#include "src/core/util/load_file.h"
+#include "src/core/util/status_helper.h"
 
 namespace grpc_core {
 
@@ -114,10 +112,9 @@ FileWatcherAuthorizationPolicyProvider::FileWatcherAuthorizationPolicyProvider(
       }
       absl::Status status = provider->ForceUpdate();
       if (GRPC_TRACE_FLAG_ENABLED(grpc_authz_api) && !status.ok()) {
-        gpr_log(GPR_ERROR,
-                "authorization policy reload status. code=%d error_details=%s",
-                static_cast<int>(status.code()),
-                std::string(status.message()).c_str());
+        LOG(ERROR) << "authorization policy reload status. code="
+                   << static_cast<int>(status.code())
+                   << " error_details=" << status.message();
       }
     }
   };
@@ -168,12 +165,10 @@ absl::Status FileWatcherAuthorizationPolicyProvider::ForceUpdate() {
   if (cb_ != nullptr) {
     cb_(contents_changed, absl::OkStatus());
   }
-  if (GRPC_TRACE_FLAG_ENABLED(grpc_authz_api)) {
-    gpr_log(GPR_INFO,
-            "authorization policy reload status: successfully loaded new "
-            "policy\n%s",
-            file_contents_.c_str());
-  }
+  GRPC_TRACE_LOG(grpc_authz_api, INFO)
+      << "authorization policy reload status: successfully loaded new "
+         "policy\n"
+      << file_contents_;
   return absl::OkStatus();
 }
 
