@@ -12,10 +12,11 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include <grpc/support/port_platform.h>
-
 #include "src/core/lib/security/authorization/rbac_translator.h"
 
+#include <grpc/grpc_audit_logging.h>
+#include <grpc/support/json.h>
+#include <grpc/support/port_platform.h>
 #include <stddef.h>
 
 #include <algorithm>
@@ -25,20 +26,26 @@
 #include <utility>
 #include <vector>
 
+#include "absl/log/check.h"
 #include "absl/status/status.h"
+#include "absl/status/statusor.h"
 #include "absl/strings/match.h"
 #include "absl/strings/str_cat.h"
 #include "absl/strings/str_format.h"
+#include "absl/strings/string_view.h"
 #include "absl/strings/strip.h"
-
-#include "src/core/lib/gpr/useful.h"
-#include "src/core/lib/json/json.h"
-#include "src/core/lib/matchers/matchers.h"
+#include "src/core/lib/security/authorization/audit_logging.h"
+#include "src/core/util/json/json.h"
+#include "src/core/util/json/json_reader.h"
+#include "src/core/util/matchers.h"
+#include "src/core/util/useful.h"
 
 namespace grpc_core {
 
 namespace {
 
+using experimental::AuditLoggerRegistry;
+
 absl::string_view GetMatcherType(absl::string_view value,
                                  StringMatcher::Type* type) {
   if (value == "*") {
@@ -90,13 +97,13 @@ bool IsUnsupportedHeader(absl::string_view header_name) {
 
 absl::StatusOr<Rbac::Principal> ParsePrincipalsArray(const Json& json) {
   std::vector<std::unique_ptr<Rbac::Principal>> principal_names;
-  for (size_t i = 0; i < json.array_value().size(); ++i) {
-    const Json& child = json.array_value().at(i);
-    if (child.type() != Json::Type::STRING) {
+  for (size_t i = 0; i < json.array().size(); ++i) {
+    const Json& child = json.array().at(i);
+    if (child.type() != Json::Type::kString) {
       return absl::InvalidArgumentError(
           absl::StrCat("\"principals\" ", i, ": is not a string."));
     }
-    auto matcher_or = GetStringMatcher(child.string_value());
+    auto matcher_or = GetStringMatcher(child.string());
     if (!matcher_or.ok()) {
       return absl::Status(matcher_or.status().code(),
                           absl::StrCat("\"principals\" ", i, ": ",
@@ -111,9 +118,9 @@ absl::StatusOr<Rbac::Principal> ParsePrincipalsArray(const Json& json) {
 
 absl::StatusOr<Rbac::Principal> ParsePeer(const Json& json) {
   std::vector<std::unique_ptr<Rbac::Principal>> peer;
-  for (const auto& object : json.object_value()) {
+  for (const auto& object : json.object()) {
     if (object.first == "principals") {
-      if (object.second.type() != Json::Type::ARRAY) {
+      if (object.second.type() != Json::Type::kArray) {
         return absl::InvalidArgumentError("\"principals\" is not an array.");
       }
       auto principal_names_or = ParsePrincipalsArray(object.second);
@@ -135,17 +142,17 @@ absl::StatusOr<Rbac::Principal> ParsePeer(const Json& json) {
 
 absl::StatusOr<Rbac::Permission> ParseHeaderValues(
     const Json& json, absl::string_view header_name) {
-  if (json.array_value().empty()) {
+  if (json.array().empty()) {
     return absl::InvalidArgumentError("\"values\" list is empty.");
   }
   std::vector<std::unique_ptr<Rbac::Permission>> values;
-  for (size_t i = 0; i < json.array_value().size(); ++i) {
-    const Json& child = json.array_value().at(i);
-    if (child.type() != Json::Type::STRING) {
+  for (size_t i = 0; i < json.array().size(); ++i) {
+    const Json& child = json.array().at(i);
+    if (child.type() != Json::Type::kString) {
       return absl::InvalidArgumentError(
           absl::StrCat("\"values\" ", i, ": is not a string."));
     }
-    auto matcher_or = GetHeaderMatcher(header_name, child.string_value());
+    auto matcher_or = GetHeaderMatcher(header_name, child.string());
     if (!matcher_or.ok()) {
       return absl::Status(
           matcher_or.status().code(),
@@ -160,19 +167,19 @@ absl::StatusOr<Rbac::Permission> ParseHeaderValues(
 absl::StatusOr<Rbac::Permission> ParseHeaders(const Json& json) {
   absl::string_view key;
   const Json* values = nullptr;
-  for (const auto& object : json.object_value()) {
+  for (const auto& object : json.object()) {
     if (object.first == "key") {
-      if (object.second.type() != Json::Type::STRING) {
+      if (object.second.type() != Json::Type::kString) {
         return absl::InvalidArgumentError("\"key\" is not a string.");
       }
-      key = object.second.string_value();
+      key = object.second.string();
       if (absl::StartsWith(key, ":") || absl::StartsWith(key, "grpc-") ||
           IsUnsupportedHeader(key)) {
         return absl::InvalidArgumentError(
             absl::StrFormat("Unsupported \"key\" %s.", key));
       }
     } else if (object.first == "values") {
-      if (object.second.type() != Json::Type::ARRAY) {
+      if (object.second.type() != Json::Type::kArray) {
         return absl::InvalidArgumentError("\"values\" is not an array.");
       }
       values = &object.second;
@@ -192,9 +199,9 @@ absl::StatusOr<Rbac::Permission> ParseHeaders(const Json& json) {
 
 absl::StatusOr<Rbac::Permission> ParseHeadersArray(const Json& json) {
   std::vector<std::unique_ptr<Rbac::Permission>> headers;
-  for (size_t i = 0; i < json.array_value().size(); ++i) {
-    const Json& child = json.array_value().at(i);
-    if (child.type() != Json::Type::OBJECT) {
+  for (size_t i = 0; i < json.array().size(); ++i) {
+    const Json& child = json.array().at(i);
+    if (child.type() != Json::Type::kObject) {
       return absl::InvalidArgumentError(
           absl::StrCat("\"headers\" ", i, ": is not an object."));
     }
@@ -212,13 +219,13 @@ absl::StatusOr<Rbac::Permission> ParseHeadersArray(const Json& json) {
 
 absl::StatusOr<Rbac::Permission> ParsePathsArray(const Json& json) {
   std::vector<std::unique_ptr<Rbac::Permission>> paths;
-  for (size_t i = 0; i < json.array_value().size(); ++i) {
-    const Json& child = json.array_value().at(i);
-    if (child.type() != Json::Type::STRING) {
+  for (size_t i = 0; i < json.array().size(); ++i) {
+    const Json& child = json.array().at(i);
+    if (child.type() != Json::Type::kString) {
       return absl::InvalidArgumentError(
           absl::StrCat("\"paths\" ", i, ": is not a string."));
     }
-    auto matcher_or = GetStringMatcher(child.string_value());
+    auto matcher_or = GetStringMatcher(child.string());
     if (!matcher_or.ok()) {
       return absl::Status(
           matcher_or.status().code(),
@@ -232,9 +239,9 @@ absl::StatusOr<Rbac::Permission> ParsePathsArray(const Json& json) {
 
 absl::StatusOr<Rbac::Permission> ParseRequest(const Json& json) {
   std::vector<std::unique_ptr<Rbac::Permission>> request;
-  for (const auto& object : json.object_value()) {
+  for (const auto& object : json.object()) {
     if (object.first == "paths") {
-      if (object.second.type() != Json::Type::ARRAY) {
+      if (object.second.type() != Json::Type::kArray) {
         return absl::InvalidArgumentError("\"paths\" is not an array.");
       }
       auto paths_or = ParsePathsArray(object.second);
@@ -244,7 +251,7 @@ absl::StatusOr<Rbac::Permission> ParseRequest(const Json& json) {
             std::make_unique<Rbac::Permission>(std::move(paths_or.value())));
       }
     } else if (object.first == "headers") {
-      if (object.second.type() != Json::Type::ARRAY) {
+      if (object.second.type() != Json::Type::kArray) {
         return absl::InvalidArgumentError("\"headers\" is not an array.");
       }
       auto headers_or = ParseHeadersArray(object.second);
@@ -269,22 +276,22 @@ absl::StatusOr<Rbac::Policy> ParseRule(const Json& json,
                                        std::string* policy_name) {
   absl::optional<Rbac::Principal> principals;
   absl::optional<Rbac::Permission> permissions;
-  for (const auto& object : json.object_value()) {
+  for (const auto& object : json.object()) {
     if (object.first == "name") {
-      if (object.second.type() != Json::Type::STRING) {
+      if (object.second.type() != Json::Type::kString) {
         return absl::InvalidArgumentError(
             absl::StrCat("\"name\" is not a string."));
       }
-      *policy_name = object.second.string_value();
+      *policy_name = object.second.string();
     } else if (object.first == "source") {
-      if (object.second.type() != Json::Type::OBJECT) {
+      if (object.second.type() != Json::Type::kObject) {
         return absl::InvalidArgumentError("\"source\" is not an object.");
       }
       auto peer_or = ParsePeer(object.second);
       if (!peer_or.ok()) return peer_or.status();
       principals = std::move(*peer_or);
     } else if (object.first == "request") {
-      if (object.second.type() != Json::Type::OBJECT) {
+      if (object.second.type() != Json::Type::kObject) {
         return absl::InvalidArgumentError("\"request\" is not an object.");
       }
       auto request_or = ParseRequest(object.second);
@@ -308,14 +315,14 @@ absl::StatusOr<Rbac::Policy> ParseRule(const Json& json,
 }
 
 absl::StatusOr<std::map<std::string, Rbac::Policy>> ParseRulesArray(
-    const Json& json, absl::string_view name) {
-  if (json.array_value().empty()) {
+    const Json& json) {
+  if (json.array().empty()) {
     return absl::InvalidArgumentError("rules is empty.");
   }
   std::map<std::string, Rbac::Policy> policies;
-  for (size_t i = 0; i < json.array_value().size(); ++i) {
-    const Json& child = json.array_value().at(i);
-    if (child.type() != Json::Type::OBJECT) {
+  for (size_t i = 0; i < json.array().size(); ++i) {
+    const Json& child = json.array().at(i);
+    if (child.type() != Json::Type::kObject) {
       return absl::InvalidArgumentError(
           absl::StrCat("rules ", i, ": is not an object."));
     }
@@ -326,55 +333,190 @@ absl::StatusOr<std::map<std::string, Rbac::Policy>> ParseRulesArray(
           policy_or.status().code(),
           absl::StrCat("rules ", i, ": ", policy_or.status().message()));
     }
-    policies[std::string(name) + "_" + policy_name] =
-        std::move(policy_or.value());
+    policies[policy_name] = std::move(policy_or.value());
   }
   return std::move(policies);
 }
 
 absl::StatusOr<Rbac> ParseDenyRulesArray(const Json& json,
                                          absl::string_view name) {
-  auto policies_or = ParseRulesArray(json, name);
+  auto policies_or = ParseRulesArray(json);
   if (!policies_or.ok()) return policies_or.status();
-  return Rbac(Rbac::Action::kDeny, std::move(policies_or.value()));
+  return Rbac(std::string(name), Rbac::Action::kDeny,
+              std::move(policies_or.value()));
 }
 
 absl::StatusOr<Rbac> ParseAllowRulesArray(const Json& json,
                                           absl::string_view name) {
-  auto policies_or = ParseRulesArray(json, name);
+  auto policies_or = ParseRulesArray(json);
   if (!policies_or.ok()) return policies_or.status();
-  return Rbac(Rbac::Action::kAllow, std::move(policies_or.value()));
+  return Rbac(std::string(name), Rbac::Action::kAllow,
+              std::move(policies_or.value()));
+}
+
+absl::StatusOr<std::unique_ptr<experimental::AuditLoggerFactory::Config>>
+ParseAuditLogger(const Json& json, size_t pos) {
+  if (json.type() != Json::Type::kObject) {
+    return absl::InvalidArgumentError(
+        absl::StrFormat("\"audit_loggers[%d]\" is not an object.", pos));
+  }
+  for (const auto& object : json.object()) {
+    if (object.first != "name" && object.first != "is_optional" &&
+        object.first != "config") {
+      return absl::InvalidArgumentError(
+          absl::StrFormat("policy contains unknown field \"%s\" in "
+                          "\"audit_logging_options.audit_loggers[%d]\".",
+                          object.first, pos));
+    }
+  }
+  bool is_optional = false;
+  auto it = json.object().find("is_optional");
+  if (it != json.object().end()) {
+    switch (it->second.type()) {
+      case Json::Type::kBoolean:
+        is_optional = it->second.boolean();
+        break;
+      default:
+        return absl::InvalidArgumentError(absl::StrFormat(
+            "\"audit_loggers[%d].is_optional\" is not a boolean.", pos));
+    }
+  }
+  it = json.object().find("name");
+  if (it == json.object().end()) {
+    return absl::InvalidArgumentError(
+        absl::StrFormat("\"audit_loggers[%d].name\" is required.", pos));
+  }
+  if (it->second.type() != Json::Type::kString) {
+    return absl::InvalidArgumentError(
+        absl::StrFormat("\"audit_loggers[%d].name\" is not a string.", pos));
+  }
+  absl::string_view name = it->second.string();
+  // The config defaults to an empty object.
+  Json config = Json::FromObject({});
+  it = json.object().find("config");
+  if (it != json.object().end()) {
+    if (it->second.type() != Json::Type::kObject) {
+      return absl::InvalidArgumentError(absl::StrFormat(
+          "\"audit_loggers[%d].config\" is not an object.", pos));
+    }
+    config = it->second;
+  }
+  if (!AuditLoggerRegistry::FactoryExists(name)) {
+    if (is_optional) {
+      return nullptr;
+    }
+    return absl::InvalidArgumentError(
+        absl::StrFormat("\"audit_loggers[%d].name\" %s is not supported "
+                        "natively or registered.",
+                        pos, name));
+  }
+  auto result = AuditLoggerRegistry::ParseConfig(name, config);
+  if (!result.ok()) {
+    return absl::InvalidArgumentError(absl::StrFormat(
+        "\"audit_loggers[%d]\" %s", pos, result.status().message()));
+  }
+  return result;
+}
+
+absl::Status ParseAuditLoggingOptions(const Json& json, RbacPolicies* rbacs) {
+  CHECK_NE(rbacs, nullptr);
+  for (auto it = json.object().begin(); it != json.object().end(); ++it) {
+    if (it->first == "audit_condition") {
+      if (it->second.type() != Json::Type::kString) {
+        return absl::InvalidArgumentError(
+            "\"audit_condition\" is not a string.");
+      }
+      absl::string_view condition = it->second.string();
+      Rbac::AuditCondition deny_condition, allow_condition;
+      if (condition == "NONE") {
+        deny_condition = Rbac::AuditCondition::kNone;
+        allow_condition = Rbac::AuditCondition::kNone;
+      } else if (condition == "ON_ALLOW") {
+        deny_condition = Rbac::AuditCondition::kNone;
+        allow_condition = Rbac::AuditCondition::kOnAllow;
+      } else if (condition == "ON_DENY") {
+        deny_condition = Rbac::AuditCondition::kOnDeny;
+        allow_condition = Rbac::AuditCondition::kOnDeny;
+      } else if (condition == "ON_DENY_AND_ALLOW") {
+        deny_condition = Rbac::AuditCondition::kOnDeny;
+        allow_condition = Rbac::AuditCondition::kOnDenyAndAllow;
+      } else {
+        return absl::InvalidArgumentError(absl::StrFormat(
+            "Unsupported \"audit_condition\" value %s.", condition));
+      }
+      if (rbacs->deny_policy.has_value()) {
+        rbacs->deny_policy->audit_condition = deny_condition;
+      }
+      rbacs->allow_policy.audit_condition = allow_condition;
+    } else if (it->first == "audit_loggers") {
+      if (it->second.type() != Json::Type::kArray) {
+        return absl::InvalidArgumentError("\"audit_loggers\" is not an array.");
+      }
+      const auto& loggers = it->second.array();
+      for (size_t i = 0; i < loggers.size(); ++i) {
+        auto result = ParseAuditLogger(loggers.at(i), i);
+        if (!result.ok()) {
+          return result.status();
+        }
+        // Check the value since the unsupported logger config can also
+        // return ok when marked as optional.
+        if (result.value() != nullptr) {
+          // Only move the logger config over if audit condition is not NONE.
+          if (rbacs->allow_policy.audit_condition !=
+              Rbac::AuditCondition::kNone) {
+            rbacs->allow_policy.logger_configs.push_back(
+                std::move(result.value()));
+          }
+          if (rbacs->deny_policy.has_value() &&
+              rbacs->deny_policy->audit_condition !=
+                  Rbac::AuditCondition::kNone) {
+            // Parse again since it returns unique_ptr, but result should be ok
+            // this time.
+            auto result = ParseAuditLogger(loggers.at(i), i);
+            CHECK(result.ok());
+            rbacs->deny_policy->logger_configs.push_back(
+                std::move(result.value()));
+          }
+        }
+      }
+    } else {
+      return absl::InvalidArgumentError(absl::StrFormat(
+          "policy contains unknown field \"%s\" in \"audit_logging_options\".",
+          it->first));
+    }
+  }
+  return absl::OkStatus();
 }
 
 }  // namespace
 
 absl::StatusOr<RbacPolicies> GenerateRbacPolicies(
     absl::string_view authz_policy) {
-  auto json = Json::Parse(authz_policy);
+  auto json = JsonParse(authz_policy);
   if (!json.ok()) {
     return absl::InvalidArgumentError(
         absl::StrCat("Failed to parse gRPC authorization policy. Error: ",
                      json.status().ToString()));
   }
-  if (json->type() != Json::Type::OBJECT) {
+  if (json->type() != Json::Type::kObject) {
     return absl::InvalidArgumentError(
         "SDK authorization policy is not an object.");
   }
-  auto it = json->object_value().find("name");
-  if (it == json->object_value().end()) {
+  auto it = json->object().find("name");
+  if (it == json->object().end()) {
     return absl::InvalidArgumentError("\"name\" field is not present.");
   }
-  if (it->second.type() != Json::Type::STRING) {
+  if (it->second.type() != Json::Type::kString) {
     return absl::InvalidArgumentError("\"name\" is not a string.");
   }
-  absl::string_view name = it->second.string_value();
+  absl::string_view name = it->second.string();
   RbacPolicies rbacs;
   bool has_allow_rbac = false;
-  for (const auto& object : json->object_value()) {
+  for (const auto& object : json->object()) {
     if (object.first == "name") {
       continue;
     } else if (object.first == "deny_rules") {
-      if (object.second.type() != Json::Type::ARRAY) {
+      if (object.second.type() != Json::Type::kArray) {
         return absl::InvalidArgumentError("\"deny_rules\" is not an array.");
       }
       auto deny_policy_or = ParseDenyRulesArray(object.second, name);
@@ -385,7 +527,7 @@ absl::StatusOr<RbacPolicies> GenerateRbacPolicies(
       }
       rbacs.deny_policy = std::move(*deny_policy_or);
     } else if (object.first == "allow_rules") {
-      if (object.second.type() != Json::Type::ARRAY) {
+      if (object.second.type() != Json::Type::kArray) {
         return absl::InvalidArgumentError("\"allow_rules\" is not an array.");
       }
       auto allow_policy_or = ParseAllowRulesArray(object.second, name);
@@ -396,11 +538,25 @@ absl::StatusOr<RbacPolicies> GenerateRbacPolicies(
       }
       rbacs.allow_policy = std::move(*allow_policy_or);
       has_allow_rbac = true;
+    } else if (object.first == "audit_logging_options") {
+      // This must be processed this after policies are all parsed.
+      continue;
     } else {
       return absl::InvalidArgumentError(absl::StrFormat(
           "policy contains unknown field \"%s\".", object.first));
     }
   }
+  it = json->object().find("audit_logging_options");
+  if (it != json->object().end()) {
+    if (it->second.type() != Json::Type::kObject) {
+      return absl::InvalidArgumentError(
+          "\"audit_logging_options\" is not an object.");
+    }
+    absl::Status status = ParseAuditLoggingOptions(it->second, &rbacs);
+    if (!status.ok()) {
+      return status;
+    }
+  }
   if (!has_allow_rbac) {
     return absl::InvalidArgumentError("\"allow_rules\" is not present.");
   }
