@@ -19,28 +19,26 @@
 #ifndef GRPC_SRC_CORE_LIB_SECURITY_CONTEXT_SECURITY_CONTEXT_H
 #define GRPC_SRC_CORE_LIB_SECURITY_CONTEXT_SECURITY_CONTEXT_H
 
+#include <grpc/credentials.h>
+#include <grpc/grpc.h>
+#include <grpc/grpc_security.h>
+#include <grpc/support/alloc.h>
+#include <grpc/support/port_platform.h>
 #include <stddef.h>
 
 #include <memory>
 #include <utility>
 
 #include "absl/strings/string_view.h"
-
-#include <grpc/credentials.h>
-#include <grpc/grpc.h>
-#include <grpc/grpc_security.h>
-#include <grpc/support/alloc.h>
-#include <grpc/support/port_platform.h>
-
 #include "src/core/lib/debug/trace.h"
-#include "src/core/lib/gpr/useful.h"
-#include "src/core/lib/gprpp/debug_location.h"
-#include "src/core/lib/gprpp/ref_counted.h"
-#include "src/core/lib/gprpp/ref_counted_ptr.h"
 #include "src/core/lib/resource_quota/arena.h"
 #include "src/core/lib/security/credentials/credentials.h"  // IWYU pragma: keep
-
-extern grpc_core::DebugOnlyTraceFlag grpc_trace_auth_context_refcount;
+#include "src/core/lib/surface/connection_context.h"
+#include "src/core/util/debug_location.h"
+#include "src/core/util/orphanable.h"
+#include "src/core/util/ref_counted.h"
+#include "src/core/util/ref_counted_ptr.h"
+#include "src/core/util/useful.h"
 
 // --- grpc_auth_context ---
 
@@ -74,13 +72,14 @@ struct grpc_auth_context
       grpc_core::RefCountedPtr<grpc_auth_context> chained)
       : grpc_core::RefCounted<grpc_auth_context,
                               grpc_core::NonPolymorphicRefCount>(
-            GRPC_TRACE_FLAG_ENABLED(grpc_trace_auth_context_refcount)
+            GRPC_TRACE_FLAG_ENABLED(auth_context_refcount)
                 ? "auth_context_refcount"
                 : nullptr),
         chained_(std::move(chained)) {
     if (chained_ != nullptr) {
       peer_identity_property_name_ = chained_->peer_identity_property_name_;
     }
+    connection_context_ = grpc_core::ConnectionContext::Create();
   }
 
   ~grpc_auth_context() {
@@ -102,6 +101,10 @@ struct grpc_auth_context
   const grpc_auth_context* chained() const { return chained_.get(); }
   const grpc_auth_property_array& properties() const { return properties_; }
 
+  grpc_core::ConnectionContext* connection_context() const {
+    return connection_context_.get();
+  }
+
   bool is_authenticated() const {
     return peer_identity_property_name_ != nullptr;
   }
@@ -124,6 +127,7 @@ struct grpc_auth_context
   grpc_auth_property_array properties_;
   const char* peer_identity_property_name_ = nullptr;
   std::unique_ptr<Extension> extension_;
+  grpc_core::OrphanablePtr<grpc_core::ConnectionContext> connection_context_;
 };
 
 // --- grpc_security_context_extension ---
@@ -136,15 +140,24 @@ struct grpc_security_context_extension {
   void (*destroy)(void*) = nullptr;
 };
 
+namespace grpc_core {
+
+class SecurityContext {
+ public:
+  virtual ~SecurityContext() = default;
+};
+
+}  // namespace grpc_core
+
 // --- grpc_client_security_context ---
 
 // Internal client-side security context.
 
-struct grpc_client_security_context {
+struct grpc_client_security_context final : public grpc_core::SecurityContext {
   explicit grpc_client_security_context(
       grpc_core::RefCountedPtr<grpc_call_credentials> creds)
       : creds(std::move(creds)) {}
-  ~grpc_client_security_context();
+  ~grpc_client_security_context() override;
 
   grpc_core::RefCountedPtr<grpc_call_credentials> creds;
   grpc_core::RefCountedPtr<grpc_auth_context> auth_context;
@@ -159,9 +172,9 @@ void grpc_client_security_context_destroy(void* ctx);
 
 // Internal server-side security context.
 
-struct grpc_server_security_context {
+struct grpc_server_security_context final : public grpc_core::SecurityContext {
   grpc_server_security_context() = default;
-  ~grpc_server_security_context();
+  ~grpc_server_security_context() override;
 
   grpc_core::RefCountedPtr<grpc_auth_context> auth_context;
   grpc_security_context_extension extension;
@@ -178,4 +191,20 @@ grpc_auth_context* grpc_auth_context_from_arg(const grpc_arg* arg);
 grpc_auth_context* grpc_find_auth_context_in_args(
     const grpc_channel_args* args);
 
+namespace grpc_core {
+template <>
+struct ArenaContextType<SecurityContext> {
+  static void Destroy(SecurityContext* p) { p->~SecurityContext(); }
+};
+
+template <>
+struct ContextSubclass<grpc_client_security_context> {
+  using Base = SecurityContext;
+};
+template <>
+struct ContextSubclass<grpc_server_security_context> {
+  using Base = SecurityContext;
+};
+}  // namespace grpc_core
+
 #endif  // GRPC_SRC_CORE_LIB_SECURITY_CONTEXT_SECURITY_CONTEXT_H
