@@ -15,6 +15,19 @@
 #ifndef GRPC_SRC_CORE_LIB_SURFACE_CLIENT_CALL_H
 #define GRPC_SRC_CORE_LIB_SURFACE_CLIENT_CALL_H
 
+#include <grpc/byte_buffer.h>
+#include <grpc/compression.h>
+#include <grpc/event_engine/event_engine.h>
+#include <grpc/grpc.h>
+#include <grpc/impl/call.h>
+#include <grpc/impl/propagation_bits.h>
+#include <grpc/slice.h>
+#include <grpc/slice_buffer.h>
+#include <grpc/status.h>
+#include <grpc/support/alloc.h>
+#include <grpc/support/atm.h>
+#include <grpc/support/port_platform.h>
+#include <grpc/support/string_util.h>
 #include <inttypes.h>
 #include <limits.h>
 #include <stdlib.h>
@@ -27,31 +40,15 @@
 #include "absl/status/status.h"
 #include "absl/strings/str_format.h"
 #include "absl/strings/string_view.h"
-
-#include <grpc/byte_buffer.h>
-#include <grpc/compression.h>
-#include <grpc/event_engine/event_engine.h>
-#include <grpc/grpc.h>
-#include <grpc/impl/call.h>
-#include <grpc/impl/propagation_bits.h>
-#include <grpc/slice.h>
-#include <grpc/slice_buffer.h>
-#include <grpc/status.h>
-#include <grpc/support/alloc.h>
-#include <grpc/support/atm.h>
-#include <grpc/support/log.h>
-#include <grpc/support/port_platform.h>
-#include <grpc/support/string_util.h>
-
-#include "src/core/lib/gprpp/crash.h"
-#include "src/core/lib/gprpp/ref_counted.h"
-#include "src/core/lib/gprpp/ref_counted_ptr.h"
-#include "src/core/lib/gprpp/single_set_ptr.h"
 #include "src/core/lib/promise/status_flag.h"
 #include "src/core/lib/resource_quota/arena.h"
 #include "src/core/lib/surface/call.h"
 #include "src/core/lib/surface/call_utils.h"
 #include "src/core/lib/transport/metadata.h"
+#include "src/core/util/crash.h"
+#include "src/core/util/ref_counted.h"
+#include "src/core/util/ref_counted_ptr.h"
+#include "src/core/util/single_set_ptr.h"
 
 namespace grpc_core {
 
@@ -99,7 +96,9 @@ class ClientCall final
   char* GetPeer() override;
 
   bool Completed() final { Crash("unimplemented"); }
-  bool failed_before_recv_message() const final { Crash("unimplemented"); }
+  bool failed_before_recv_message() const final {
+    return started_call_initiator_.WasCancelledPushed();
+  }
 
   grpc_compression_algorithm incoming_compression_algorithm() override {
     return message_receiver_.incoming_compression_algorithm();
@@ -129,7 +128,15 @@ class ClientCall final
                    bool is_notify_tag_closure);
   template <typename Batch>
   void ScheduleCommittedBatch(Batch batch);
-  void StartCall(const grpc_op& send_initial_metadata_op);
+  Party::WakeupHold StartCall(const grpc_op& send_initial_metadata_op);
+  // Attempt to start the call and send handler down the stack; returns true if
+  // state was updated, false otherwise (with cur_state updated to the new
+  // current state).
+  // If this function returns false, it's guaranteed that handler is not
+  // touched.
+  // Should be called repeatedly until it returns true.
+  bool StartCallMaybeUpdateState(uintptr_t& cur_state,
+                                 UnstartedCallHandler& handler);
 
   std::string DebugTag() { return absl::StrFormat("CLIENT_CALL[%p]: ", this); }
 
