@@ -12,10 +12,10 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include <grpc/support/port_platform.h>
-
 #include "src/core/lib/transport/parsed_metadata.h"
 
+#include <grpc/support/port_platform.h>
+
 namespace grpc_core {
 namespace metadata_detail {
 
@@ -24,12 +24,12 @@ std::string MakeDebugString(absl::string_view key, absl::string_view value) {
 }
 
 Slice SliceFromBuffer(const Buffer& buffer) {
-  return Slice(grpc_slice_ref_internal(buffer.slice));
+  return Slice(CSliceRef(buffer.slice));
 }
 
-void DestroySliceValue(const Buffer& value) {
-  grpc_slice_unref_internal(value.slice);
-}
+void DestroySliceValue(const Buffer& value) { CSliceUnref(value.slice); }
+
+void DestroyTrivialMemento(const Buffer&) {}
 
 }  // namespace metadata_detail
 }  // namespace grpc_core
