@@ -14,6 +14,7 @@
 // limitations under the License.
 //
 
+#include <grpc/impl/channel_arg_names.h>
 #include <grpc/support/port_platform.h>
 
 #include <algorithm>
@@ -22,31 +23,28 @@
 #include <vector>
 
 #include "absl/functional/bind_front.h"
+#include "absl/log/log.h"
 #include "absl/status/status.h"
 #include "absl/status/statusor.h"
 #include "absl/strings/str_cat.h"
 #include "absl/strings/string_view.h"
 #include "absl/strings/strip.h"
 #include "absl/types/optional.h"
-
-#include <grpc/impl/channel_arg_names.h>
-#include <grpc/support/log.h>
-
-#include "src/core/resolver/polling_resolver.h"
-#include "src/core/lib/backoff/backoff.h"
+#include "src/core/config/core_configuration.h"
 #include "src/core/lib/channel/channel_args.h"
-#include "src/core/lib/config/core_configuration.h"
 #include "src/core/lib/debug/trace.h"
-#include "src/core/lib/gprpp/debug_location.h"
-#include "src/core/lib/gprpp/orphanable.h"
-#include "src/core/lib/gprpp/ref_counted_ptr.h"
-#include "src/core/lib/gprpp/time.h"
 #include "src/core/lib/iomgr/resolve_address.h"
 #include "src/core/lib/iomgr/resolved_address.h"
 #include "src/core/resolver/endpoint_addresses.h"
+#include "src/core/resolver/polling_resolver.h"
 #include "src/core/resolver/resolver.h"
 #include "src/core/resolver/resolver_factory.h"
-#include "src/core/lib/uri/uri_parser.h"
+#include "src/core/util/backoff.h"
+#include "src/core/util/debug_location.h"
+#include "src/core/util/orphanable.h"
+#include "src/core/util/ref_counted_ptr.h"
+#include "src/core/util/time.h"
+#include "src/core/util/uri.h"
 
 #define GRPC_DNS_INITIAL_CONNECT_BACKOFF_SECONDS 1
 #define GRPC_DNS_RECONNECT_BACKOFF_MULTIPLIER 1.6
@@ -57,9 +55,7 @@ namespace grpc_core {
 
 namespace {
 
-TraceFlag grpc_trace_dns_resolver(false, "dns_resolver");
-
-class NativeClientChannelDNSResolver : public PollingResolver {
+class NativeClientChannelDNSResolver final : public PollingResolver {
  public:
   NativeClientChannelDNSResolver(ResolverArgs args,
                                  Duration min_time_between_resolutions);
@@ -71,7 +67,7 @@ class NativeClientChannelDNSResolver : public PollingResolver {
   // No-op request class, used so that the PollingResolver code knows
   // when there is a request in flight, even if the request is not
   // actually cancellable.
-  class Request : public Orphanable {
+  class Request final : public Orphanable {
    public:
     Request() = default;
 
@@ -92,16 +88,12 @@ NativeClientChannelDNSResolver::NativeClientChannelDNSResolver(
                           .set_jitter(GRPC_DNS_RECONNECT_JITTER)
                           .set_max_backoff(Duration::Milliseconds(
                               GRPC_DNS_RECONNECT_MAX_BACKOFF_SECONDS * 1000)),
-                      &grpc_trace_dns_resolver) {
-  if (GRPC_TRACE_FLAG_ENABLED(grpc_trace_dns_resolver)) {
-    gpr_log(GPR_DEBUG, "[dns_resolver=%p] created", this);
-  }
+                      &dns_resolver_trace) {
+  GRPC_TRACE_VLOG(dns_resolver, 2) << "[dns_resolver=" << this << "] created";
 }
 
 NativeClientChannelDNSResolver::~NativeClientChannelDNSResolver() {
-  if (GRPC_TRACE_FLAG_ENABLED(grpc_trace_dns_resolver)) {
-    gpr_log(GPR_DEBUG, "[dns_resolver=%p] destroyed", this);
-  }
+  GRPC_TRACE_VLOG(dns_resolver, 2) << "[dns_resolver=" << this << "] destroyed";
 }
 
 OrphanablePtr<Orphanable> NativeClientChannelDNSResolver::StartRequest() {
@@ -110,19 +102,17 @@ OrphanablePtr<Orphanable> NativeClientChannelDNSResolver::StartRequest() {
       absl::bind_front(&NativeClientChannelDNSResolver::OnResolved, this),
       name_to_resolve(), kDefaultSecurePort, kDefaultDNSRequestTimeout,
       interested_parties(), /*name_server=*/"");
-  if (GRPC_TRACE_FLAG_ENABLED(grpc_trace_dns_resolver)) {
-    gpr_log(GPR_DEBUG, "[dns_resolver=%p] starting request=%p", this,
-            DNSResolver::HandleToString(dns_request_handle).c_str());
-  }
+  GRPC_TRACE_VLOG(dns_resolver, 2)
+      << "[dns_resolver=" << this << "] starting request="
+      << DNSResolver::HandleToString(dns_request_handle);
   return MakeOrphanable<Request>();
 }
 
 void NativeClientChannelDNSResolver::OnResolved(
     absl::StatusOr<std::vector<grpc_resolved_address>> addresses_or) {
-  if (GRPC_TRACE_FLAG_ENABLED(grpc_trace_dns_resolver)) {
-    gpr_log(GPR_DEBUG, "[dns_resolver=%p] request complete, status=\"%s\"",
-            this, addresses_or.status().ToString().c_str());
-  }
+  GRPC_TRACE_VLOG(dns_resolver, 2)
+      << "[dns_resolver=" << this
+      << "] request complete, status=" << addresses_or.status();
   // Convert result from iomgr DNS API into Resolver::Result.
   Result result;
   if (addresses_or.ok()) {
@@ -145,17 +135,17 @@ void NativeClientChannelDNSResolver::OnResolved(
 // Factory
 //
 
-class NativeClientChannelDNSResolverFactory : public ResolverFactory {
+class NativeClientChannelDNSResolverFactory final : public ResolverFactory {
  public:
   absl::string_view scheme() const override { return "dns"; }
 
   bool IsValidUri(const URI& uri) const override {
     if (GPR_UNLIKELY(!uri.authority().empty())) {
-      gpr_log(GPR_ERROR, "authority based dns uri's not supported");
+      LOG(ERROR) << "authority based dns uri's not supported";
       return false;
     }
     if (absl::StripPrefix(uri.path(), "/").empty()) {
-      gpr_log(GPR_ERROR, "no server name supplied in dns URI");
+      LOG(ERROR) << "no server name supplied in dns URI";
       return false;
     }
     return true;
